package com.yeejoin.amos.fas.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.yeejoin.amos.fas.business.constants.FasConstant;
import com.yeejoin.amos.fas.business.dao.mapper.*;
import com.yeejoin.amos.fas.business.dao.repository.*;
import com.yeejoin.amos.fas.business.feign.IMaasVisualServer;
import com.yeejoin.amos.fas.business.feign.PushFeign;
import com.yeejoin.amos.fas.business.jpush.JPushTypeEnum;
import com.yeejoin.amos.fas.business.jpush.PushMsgParam;
import com.yeejoin.amos.fas.business.service.intfc.IDictService;
import com.yeejoin.amos.fas.business.service.intfc.IPlanVisual3dService;
import com.yeejoin.amos.fas.business.service.model.ToipResponse;
import com.yeejoin.amos.fas.business.util.JSONUtil;
import com.yeejoin.amos.fas.business.util.StringUtil;
import com.yeejoin.amos.fas.business.vo.PlanStepVo;
import com.yeejoin.amos.fas.business.vo.TreeSubjectVo;
import com.yeejoin.amos.fas.core.util.CommonResponse;
import com.yeejoin.amos.fas.core.util.CommonResponseUtil;
import com.yeejoin.amos.fas.core.util.DateUtil;
import com.yeejoin.amos.fas.dao.entity.*;
import com.yeejoin.amos.feign.privilege.model.AgencyUserModel;
import com.yeejoin.amos.feign.privilege.model.RoleModel;
import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.Resource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;

@Service("planVisual3dService")
public class PlanVisual3dServiceImpl implements IPlanVisual3dService {

    @Autowired
    IMaasVisualServer maasVisualServer;

    @Autowired
    ITextPlanDao iTextPlanDao;

    @Autowired
    PlanVisual3dMapper planVisual3dMapper;

    @Autowired
    IDictService dictService;

    @Autowired
    private DictMapper dictMapper;

    @Autowired
    PlanOperationRecordMapper planOperationRecordMapper;

    @Autowired
    IContingencyOriginalDataDao contingencyOriginalDataDao;

    @Autowired
    private IContingencyPlanInstanceRepository contingencyPlanInstanceRepository;

    @Autowired
    private IPlanRuleDao planRuleDao;

    @Autowired
    private IPlanOperationRecordDao planOperationRecordDao;

    @Autowired
    private ContingencyInstanceInfoMapper contingencyInstanceInfoMapper;

    @Autowired
    IContingencyPlanInstanceRepository iContingencyPlanInstanceRepository;


    @Autowired
    private ContingencyPlanInstanceMapper contingencyPlanInstanceMapper;

    @Value("classpath:/json/plan-step.json")
    private Resource planStepResource;

    @Autowired
    private PushFeign pushFeign;

    @Autowired
    private IContingencyPlanInstanceRepository repository;


    @Override
    public List<TreeSubjectVo> getPlanTree() {

        CommonResponse response = maasVisualServer.getTree();
        List<TreeSubjectVo> treeSubjectVos = Lists.newArrayList();
        if (null != response.getDataList()) {
            List<Object> res = (List<Object>) response.getDataList();
            if (!CollectionUtils.isEmpty(res)) {
                treeSubjectVos = listToTree(res);
            }
        }
        return treeSubjectVos;
    }

    @Override
    public void uploadTextPlan(String appId, Map<String, String> pathNameMap) {
        if (null != pathNameMap && !pathNameMap.isEmpty()) {
            for (Map.Entry<String, String> entry : pathNameMap.entrySet()) {
                TextPlan textPlan = new TextPlan();
                textPlan.setAppId(appId);
                textPlan.setTextName(entry.getValue());
                textPlan.setFilePath(entry.getKey());
                iTextPlanDao.save(textPlan);
            }
        }
    }

    @Override
    public TextPlan getTextPlanInfoById(Long id) {
        Optional<TextPlan> textPlan = iTextPlanDao.findById(id);
        if (textPlan.isPresent()) {
            return textPlan.get();
        }
        return null;
    }

    private List<TreeSubjectVo> listToTree(List<Object> list) {
        List<TreeSubjectVo> treeSubjectVos = Lists.newArrayList();
        for (Object obj : list) {
            TreeSubjectVo treeItem = new TreeSubjectVo();
            LinkedHashMap<String, Object> map = (LinkedHashMap<String, Object>) obj;
            treeItem.setId((String) map.get("id"));
            treeItem.setTreeName((String) map.get("treeName"));
            treeItem.setParentId((String) map.get("parentId"));
            treeItem.setMaxSubjectNumber((Integer) map.get("maxSubjectNumber"));
            treeItem.setType("treeNode");

            List<Object> children = (List<Object>) map.get("children");
            if (null != children) {
                List<TreeSubjectVo> subjectVos = listToTree(children);
                treeItem.setChildren(subjectVos);
            }

            CommonResponse response = maasVisualServer.getSubjectById(String.valueOf(map.get("id")));
            if (null != response) {
                List<TreeSubjectVo> childrenVos = Lists.newArrayList();
                List<Object> res = (List<Object>) response.getDataList();
                res.forEach(r -> {
                    LinkedHashMap<String, Object> map1 = (LinkedHashMap<String, Object>) r;
                    Integer state = (Integer) map1.get("publishState");
                    if (state == 0) {//过滤 未发布状态
                        return;
                    }
                    TreeSubjectVo subjectVo = new TreeSubjectVo();
                    subjectVo.setType("listNode");
                    subjectVo.setParentId((String) map.get("id"));
                    subjectVo.setId((String) map1.get("id"));
                    subjectVo.setDelete(map1.get("isDelete") == null || (Integer) map1.get("isDelete") == 0 ? false : true);
                    subjectVo.setCreateTime((String) map1.get("createTime"));
                    subjectVo.setCreator(map1.get("creator") == null ? 0 : (Integer) map1.get("creator"));
                    subjectVo.setSubjectName((String) map1.get("subjectName"));
                    subjectVo.setTreeName((String) map1.get("subjectName"));
                    subjectVo.setSubjectDetail((String) map1.get("subjectDetail"));
                    subjectVo.setSubjectType((String) map1.get("subjectType"));
                    subjectVo.setSubjectTypeName((String) map1.get("subjectTypeName"));
                    subjectVo.setPublishState(state);
                    subjectVo.setBelongTreeId((String) map1.get("belongTreeId"));
                    subjectVo.setScenes(map1.get("scenes") == null ? "" : (String) map1.get("scenes"));

                    //关联文字预案
                    List<TextPlan> textPlans = iTextPlanDao.findByAppId(String.valueOf(subjectVo.getId()));
                    textPlans.forEach(t -> {
                        TreeSubjectVo textTreeSubjectVo = new TreeSubjectVo();
                        textTreeSubjectVo.setParentId(String.valueOf(t.getAppId()));
                        textTreeSubjectVo.setTreeName(t.getTextName());
                        textTreeSubjectVo.setId(String.valueOf(t.getId()));
                        textTreeSubjectVo.setType("textNode");
                        subjectVo.getChildren().add(textTreeSubjectVo);
                    });

                    childrenVos.add(subjectVo);
                });
                treeItem.getChildren().addAll(childrenVos);
            }
            treeSubjectVos.add(treeItem);
        }
        return treeSubjectVos;
    }

    @Override
    public CommonResponse getResourceListByType(String type) {
        //1.校验是否是约定的字典，不存在则提示失败
        Dict dict = new Dict();
        dict.setDictCode(FasConstant.PLAN_SOURCE_TYPE);
        dict.setDictValue(type);
        List<Dict> dictList = dictService.getDictList(dict);
        if (CollectionUtils.isEmpty(dictList)) {
            return CommonResponseUtil.failure(type + "字典类型不存在");
        }
        //2.返回存在的数据
//        return CommonResponseUtil.success(planVisual3dMapper.getResourceListByType(type));
        return CommonResponseUtil.success();
    }

    @Override
    public List<Map<String, Object>> getResourceById(String type, Long id) {
        //1.查询出要返回数据
//        Map<String, Object> map = planVisual3dMapper.queryOneByTypeAndId(type, id);
        Map<String, Object> map = new HashMap<String, Object>();
        //1.1无数据则返回
        if (CollectionUtils.isEmpty(map)) {
            return new ArrayList<>();
        }
        //2.加工数据
        //2.0查询出中英文对照字典映射
        Dict dict = new Dict();
        dict.setDictCode(type);
        List<Dict> dictList = dictService.getDictList(dict);
        Map<String, String> dicts = dictList.stream().collect(Collectors.toMap(Dict::getDictValue, Dict::getDictName, (key1, key2) -> key2));

        //2.1换key为中文
        Map<String, Object> tempMap = new HashMap<String, Object>();
        map.forEach((k, v) -> {
            if (dicts.containsKey(k)) {
                tempMap.put(dicts.get(k), v);
            }
        });

        //2.2map转list
        List<Map<String, Object>> list = new ArrayList<>();
        list = tempMap.entrySet().stream().map(e -> {
            Map<String, Object> newMap = new HashMap<String, Object>();
            newMap.put("label", e.getKey());
            newMap.put("value", e.getValue());
            return newMap;
        }).collect(Collectors.toList());
        return list;
    }

    @Override
    public List<Map<String, Object>> getResourceTypeList() {
        Dict dict = new Dict();
        dict.setDictCode(FasConstant.PLAN_SOURCE_TYPE);
        List<Dict> dictList = dictMapper.getDictList(dict);
        List<Map<String, Object>> list = new ArrayList<>();
        list = dictList.stream().map(e -> {
            Map<String, Object> newMap = new HashMap<String, Object>();
            newMap.put("key", e.getDictValue());
            newMap.put("value", e.getDictValue());//前端级联Cascader使用
            newMap.put("isLeaf", false);////前端级联Cascader使用
            newMap.put("dataType", "type");
            newMap.put("label", e.getDictName());
            return newMap;
        }).collect(Collectors.toList());
        return list;
    }

    @Override
    public List<TreeSubjectVo> getTextPlanBySubjectId(String appId) {
        //关联文字预案
        List<TreeSubjectVo> subjectVos = new ArrayList<>();
        List<TextPlan> textPlans = iTextPlanDao.findByAppId(appId);
        textPlans.forEach(t -> {
            TreeSubjectVo textTreeSubjectVo = new TreeSubjectVo();
            textTreeSubjectVo.setParentId(String.valueOf(t.getAppId()));
            textTreeSubjectVo.setTreeName(t.getTextName());
            textTreeSubjectVo.setId(String.valueOf(t.getId()));
            textTreeSubjectVo.setType("textNode");
            subjectVos.add(textTreeSubjectVo);
        });
        return subjectVos;
    }

    @Override
    public void deleteTextPlanFile(Long id) {
        iTextPlanDao.deleteById(id);
    }

    @Override
    public String getNewestBatchNo() {
        return planOperationRecordMapper.getNewestBatchNo();
    }

    @Override
    public ToipResponse getPlanStepByBatchNo(String batchNo) {
        ToipResponse toipResponse = new ToipResponse();
        // 根据批次号查询预案步骤
        String planStep = planOperationRecordMapper.getPlanStepByBatchNo(batchNo);
        if (StringUtil.isNotEmpty(planStep)) {
            JSONArray objects = JSON.parseArray(planStep);
            // 查询当前步骤
            ContingencyOriginalData contingencyOriginalData = contingencyOriginalDataDao.findByBatchNo(batchNo);
            String step = contingencyOriginalData.getStep();
            String confirm = contingencyOriginalData.getConfirm();
            Map<String, Object> msgContext = new HashMap<>(2);
            msgContext.put("step", objects);
            msgContext.put("currentStep", contingencyOriginalData.getStep());
            msgContext.put("confirm", contingencyOriginalData.getConfirm());
            msgContext.put("createDate", contingencyOriginalData.getCreateDate());
            List<ContingencyPlanInstance> list = contingencyPlanInstanceRepository.findByBatchNo(batchNo);
            if (!CollectionUtils.isEmpty(list)) {
                List<ContingencyPlanInstance> collect = list.stream().filter(x -> "OPERATE".equals(x.getRecordType())).collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(collect)) {
                    msgContext.put("caseId", collect.get(0).getId());
                }
            }
            toipResponse.setMsgType("steparea");
            toipResponse.setMsgContext(msgContext);
            toipResponse.setContingency(contingencyOriginalData);
        }
        return toipResponse;
    }

    @Override
    public ToipResponse getPlaneRecordByBatchNo(String batchNo) {
        // 根据批次号获取预案记录
        List<Map<String, Object>> instancesList = contingencyPlanInstanceRepository.queryRecord(batchNo, "MESSAGE");
        ToipResponse toipResponse = new ToipResponse();
        if (!ObjectUtils.isEmpty(instancesList)) {
            Map<String, Object> msgContext = new HashMap<>(1);
            msgContext.put("content", instancesList);
            ContingencyOriginalData contingencyOriginalData = contingencyOriginalDataDao.findByBatchNo(batchNo);
            toipResponse.setMsgType("recordarea");
            toipResponse.setMsgContext(msgContext);
            toipResponse.setContingency(contingencyOriginalData);
        }
        return toipResponse;
    }

    @Override
    public ToipResponse getPlaneRecordOneByBatchNo(String batchNo) {
        // 根据批次号获取预案记录
        String instances = contingencyPlanInstanceRepository.queryRecordOne(batchNo, "HELPAREA");
        ToipResponse toipResponse = new ToipResponse();
        if (StringUtil.isNotEmpty(instances)) {
            Map<String, Object> stringObjectMap = JSONUtil.toMap(instances);
            ContingencyOriginalData contingencyOriginalData = contingencyOriginalDataDao.findByBatchNo(batchNo);
            toipResponse.setMsgType("helparea");
            toipResponse.setMsgContext(stringObjectMap);
            toipResponse.setContingency(contingencyOriginalData);
        }
        return toipResponse;
    }

    @Override
    @Transactional
    public PlanStepVo updatePlanStep(PlanStepVo planStepVo) {
        // 运行中的设置为重置
        List<PlanOperationRecord> planList = planOperationRecordDao.findByStatus(0);
        if (!CollectionUtils.isEmpty(planList)) {
            // 根据批次号查询预案步骤
            PlanRule planRule = planOperationRecordMapper.getPlanRuleByBatchNo(planStepVo.getBatchNo());
            planRule.setPlanStep(planStepVo.getPlanStep());
            planRuleDao.save(planRule);
        }
        return planStepVo;
    }

    @Override
    public ContingencyInstanceInfo selectDisposalDetails(String disposalId) {
        return contingencyInstanceInfoMapper.selectDisposalDetails(disposalId);
    }

    @Override
    public Page<ContingencyInstanceInfo> selectDisposalListPage(int current, int size, String orgCode) {
        Page<ContingencyInstanceInfo> page = new Page<>(current, size);
        int total = contingencyInstanceInfoMapper.selectCount(orgCode);
        page.setTotal(total);
        long start = (page.getCurrent() - 1) * page.getSize();
        if (total == 0) {
            page.setCurrent(1);
        } else {
            if (total < start) {
                page.setCurrent(1);
                start = 0;
            }
            List<ContingencyInstanceInfo> infoList = contingencyInstanceInfoMapper.selectDisposalListPage((int) start, size, orgCode);
            page.setRecords(infoList);
        }
        return page;
    }

    @Override
    public Boolean addDisposalDetails(ContingencyInstanceInfo contingencyInstanceInfo) {
        contingencyInstanceInfo.setIsDelete(false);
        contingencyInstanceInfo.setCreateDate(DateUtil.getDateNow());
        return contingencyInstanceInfoMapper.addDisposalDetails(contingencyInstanceInfo);
    }

    @Override
    public Boolean updateDisposalDetails(ContingencyInstanceInfo contingencyInstanceInfo) {
        return contingencyInstanceInfoMapper.updateDisposalDetails(contingencyInstanceInfo);
    }

    @Override
    public Page<ContingencyPlanInstance> getPlaneRecordByNew(Integer pageNum, Integer size, String batchNo) {
        Integer planInstanceCount = contingencyPlanInstanceMapper.getPlanInstanceCount(batchNo);
        // 根据批次号获取预案记录
        List<ContingencyPlanInstance> planInstanceListByPage = contingencyPlanInstanceMapper.getPlanInstanceListByPage(batchNo, (pageNum - 1) * size, size);
        if (!CollectionUtils.isEmpty(planInstanceListByPage)) {
            planInstanceListByPage.stream().sorted(Comparator.comparing(ContingencyPlanInstance::getCreateDate)).collect(Collectors.toList());
        }
        Page<ContingencyPlanInstance> page = new Page<>();
        page.setTotal(planInstanceCount);
        page.setCurrent(pageNum);
        page.setSize(size);
        page.setRecords(planInstanceListByPage);
        return page;
    }

    @Override
    public void replyMessage(AgencyUserModel user, ContingencyPlanInstance dto) {
        String json = "";
        try {
            json = IOUtils.toString(planStepResource.getInputStream(), String.valueOf(StandardCharsets.UTF_8));
        } catch (IOException e) {
            throw new RuntimeException("获取预案相关角色信息失败！");
        }
        List<Object> list = JSON.parseArray(json);
        List<Map<String, String>> result = new ArrayList<Map<String, String>>();
        for (Object object : list) {
            Map<String, String> ret = (Map<String, String>) object;
            result.add(ret);
        }
        List<String> roleCode = result.stream().map(t -> t.get("roleCode")).distinct().collect(Collectors.toList());

        List<RoleModel> roleModelList = new ArrayList<>();
        for (Long key : user.getOrgRoles().keySet()) {
            roleModelList.addAll(user.getOrgRoles().get(key));
        }
        List<String> roleCodeList = new ArrayList<>();
        List<String> roleNameList = new ArrayList<>();
        List<String> realRoleCode = roleModelList.stream().map(RoleModel::getRoleName).distinct().collect(Collectors.toList());
        realRoleCode.forEach(item -> {
            if (roleCode.contains(item)) {
                roleCodeList.add(item);
            }
        });
        roleModelList.forEach(item -> {
            if (!roleNameList.contains(item.getRoleDesc()) && roleCode.contains(item.getRoleName())) {
                roleNameList.add(item.getRoleDesc());
            }
        });
        dto.setRoleCode(String.join(",", roleCodeList));
        dto.setRoleName(String.join(",", roleNameList));
        dto.setStartUserId(user.getUserId());
        dto.setStartUserId(user.getRealName());
        dto.setRecordType("REPLYMESSAGE");
        Map<String, String> userInfo = contingencyPlanInstanceMapper.getUserByUserId(user.getUserId());
        dto.setPersonImg(userInfo.get("personImg"));
        repository.save(dto);
        PushMsgParam pushMsgParam = new PushMsgParam();
        // TODO: 2022/10/18  接收人需从平台获取
        pushMsgParam.setRecivers(Lists.newArrayList(""));
        pushMsgParam.setContent(JSONUtil.toJson(dto));
        pushMsgParam.setSubject("REPLYMESSAGE");
        pushMsgParam.setType(JPushTypeEnum.REPLY_MESSAGE.getCode());
        pushFeign.sendMessageone(pushMsgParam);
    }

    @Override
    public Page<ContingencyPlanInstance> selectDisposalActionPage(int current, int size, String disposalId, List<RoleModel> roleModelList, int dataType) {
        List<String> roles = new ArrayList<>();
        String type = "OPERATE";
        String status = "";
        Page<ContingencyPlanInstance> page = new Page<>(current, size);
        if ("1".equals(String.valueOf(dataType))) {
            for (RoleModel roleModel : roleModelList) {
                roles.add(roleModel.getRoleName());
            }
            // 根据当前用户预案角色、未执行动作
            status = "1";
        } else if ("2".equals(String.valueOf(dataType))) {
            // 当前用户预案角色、全部动作数据
            for (RoleModel roleModel : roleModelList) {
                roles.add(roleModel.getRoleName());
            }
        }
        int total = 0;
        if (ObjectUtils.isEmpty(status)) {
            total = contingencyInstanceInfoMapper.selectCountDisposalActionPage(type, null, roles, disposalId);
        } else {
            total = contingencyInstanceInfoMapper.selectCountDisposalActionPage(type, status, roles, disposalId);

        }
        long start = (page.getCurrent() - 1) * page.getSize();
        if (total == 0) {
            page.setCurrent(1);
        } else {
            if (total < start) {
                page.setCurrent(1);
                start = 0;
            }
            List<ContingencyPlanInstance> list = contingencyInstanceInfoMapper.selectDisposalActionPage((int) start, size, type, status, roles, disposalId);
            page.setRecords(list);
            page.setTotal(total);
        }
        return page;
    }

    @Override
    public List<ContingencyPlanInstance> selectDisposalActionList(String disposalId, List<RoleModel> roleModelList, int dataType) {
        List<String> roles = new ArrayList<>();
        String type = "OPERATE";
        String status = "";
        if ("1".equals(String.valueOf(dataType))) {
            for (RoleModel roleModel : roleModelList) {
                roles.add(roleModel.getRoleName());
            }
            // 根据当前用户预案角色、未执行动作
            status = "1";
        } else if ("2".equals(String.valueOf(dataType))) {
            // 当前用户预案角色、全部动作数据
            for (RoleModel roleModel : roleModelList) {
                roles.add(roleModel.getRoleName());
            }
        }
        return contingencyInstanceInfoMapper.selectDisposalActionList(type, status, roles, disposalId);
    }
}
